/******************************************************************************
**
**	Apple Macintosh Developer Technical Support
**
**	Sessions is a System 7.0 and Communications Toolbox aware sample
**
**	Copyright  1989 Apple Computer, Inc.
**	All rights reserved.
**
**	Author:		James Beninghaus
**	Versions:	1.00				1/91
**
**	Appl.MenuCommand.c contains code to process menu selections by the user.
**	The main routine found here is MenuCommand which dispatches to various
**	utility routines to perfrom the individual functions. These utility
**	routines are also called in response to required AppleEvents by the 
**	appropriate AppleEvent handler.
**
******************************************************************************/

#include	<Sessions.h>

/* 
**	DoNewSession creates a  untitled new session based on what ever CTB Tools are found
**	available in the System. This implies that the Connection, File Transfer
**	and Terminal choices and settings will be arbitrary. This routine is
**	only called when DoPreferedSession has failed.
**	The session record for this new session is pulled from the available free list
**	of sessions.
*/
OSErr DoNewSession(SessionPtr *newSession) {
	auto		OSErr			osErr;
	auto		SessionPtr		session;
	
	if (noErr == (osErr = ((nil == (session = AvailSession())) ? kUnknownError : noErr))) {
		*newSession = session;
		if (noErr == (osErr = SessionNew(session))) {
			SetWTitle((WindowPtr) &(*session).window, "\puntitled");
			(*session).alias = nil;
			ShowWindow(&(*session).window.port);
		}
	}
	return osErr;
}

/* 
**	DoPreferedSession creates a new untitled session based on the choices and settings
**	in the Preference File.
**	The session record for this new session is pulled from the available free list
**	of sessions.
*/
OSErr DoPreferedSession(SessionPtr *newSession) {
	auto		OSErr			osErr;
	auto		short			prefRefNum;
	auto		SessionPtr		session;
	auto		FSSpec			file;
	
	if (noErr == (osErr = FindPreferenceFile(&file))) {
		prefRefNum = FSpOpenResFile(&file, fsRdPerm);	
		if (-1 == prefRefNum)
			osErr = ResError();
		else {
			if (noErr == (osErr = ((nil == (session = AvailSession())) ? kUnknownError : noErr))) {
				*newSession = session;
				if (noErr == (osErr = SessionOpen(session, prefRefNum))) {
					SetWTitle((WindowPtr) &(*session).window, "\puntitled");
					(*session).alias = nil;
					ShowWindow(&(*session).window.port);
				}			}
			CloseResFile(prefRefNum);
		}
	}
	return osErr;
}

/*
**	DoOpenSession creates a new session based on a particular document file.
**	Since the resulting session is attached to a document file the session
**	window will have the same title as the document. The resulting session will
**	have an alias record attaching which attaches this new session to the document.
**	If the document file is stationary then no alias is attached.
**	The session record for this new session is pulled from the available free list
**	of sessions.
*/
OSErr DoOpenSession(SessionPtr *newSession, FSSpecPtr file, Boolean stationary) {
	auto		OSErr		osErr;
	auto		short		docRefNum;
	auto		SessionPtr	session;
	
	docRefNum = FSpOpenResFile(file, fsRdPerm);	
	if (-1 == docRefNum)
		osErr = ResError();
	else {
		if (noErr == (osErr = ((nil == (session = AvailSession())) ? kUnknownError : noErr))) {
			*newSession = session;
			if (noErr == (osErr = SessionOpen(session, docRefNum))) {
				if (stationary)  {
					(*session).alias = nil;
					BlockMove(".copy", &(*file).name[(*file).name[0] + 1],  5);
					(*file).name[0] += 5;
					SetWTitle((WindowPtr) &(*session).window, (*file).name);
				} else {
					osErr = NewAlias(nil, file, &(*session).alias);
					SetWTitle((WindowPtr) &(*session).window, (*file).name);
				}
				ShowWindow(&(*session).window.port);
			}
		}
		CloseResFile(docRefNum);
	}
	return osErr;
}

/*
**	DoSaveSession creates a resource file, opens it, writes the session
**	to the file, writes the Application-unavailable string to the file 
**	and closes the file.
*/
OSErr DoSaveSession(SessionPtr session, FSSpecPtr file) {
	auto		OSErr		osErr;
	auto		short		docRefNum;
	auto		Handle		h;
	
	FSpCreateResFile(file, kCreatorType, kDocFileType, kScriptType);
	if (noErr == (osErr = ResError())) {
		docRefNum = FSpOpenResFile(file, fsRdWrPerm);	
		if (-1 == docRefNum)
			osErr = ResError();
		else {
			if (noErr == (osErr = SessionSave(session, docRefNum)))
				if (nil == (h = Get1Resource('STR ', -16396)))
					osErr = AddResourcePtr(kSessionApplString, sizeof(kSessionApplString), 'STR ', -16396, "");
			CloseResFile(docRefNum);
		}
	}
	return osErr;
}

/*
**	DoSafeSaveSession creates a specification for a temporary file in the Temorary 
**	Folder, saves the session to this temporary file and then exchanges the data
**	in the temporary file with the existing target file.
*/
OSErr DoSafeSaveSession(SessionPtr session, FSSpecPtr file) {
	auto		OSErr		osErr;
	auto		Str63		tempName;
	auto		short		tempVRefNum;
	auto		long		tempDirID;
	auto		FSSpec		tempFSSpec;
	auto		FInfo		fndrInfo;


	if (noErr == (osErr = FindFolder((*file).vRefNum, kTemporaryFolderType, kCreateFolder, &tempVRefNum, &tempDirID))) {
		do 
			GetTempFileName(&tempName);
		while (fnfErr != (osErr = FSMakeFSSpec(tempVRefNum, tempDirID, tempName, &tempFSSpec)));
		if (fnfErr == osErr) {
			if (noErr == (osErr = DoSaveSession(session, &tempFSSpec))) {
				if (noErr == (osErr = FSpExchangeFiles(&tempFSSpec, file))) {
					if (noErr == (osErr = FSpGetFInfo(file, &fndrInfo))) {
						fndrInfo.fdCreator	= kCreatorType;
						fndrInfo.fdType		= kDocFileType;
						fndrInfo.fdFlags	= 0;
						if (noErr == (osErr = FSpSetFInfo(file, &fndrInfo))) {
							if (noErr == (osErr = FSpDelete(&tempFSSpec))) {
								osErr = noErr;
							}
						}
					}
				}
			}
		}
	}
	return osErr;
}

/*
**	DoCloseSession terminiates the session by closing the connection, deallocating the
**	CTB data and finally adding the session record back into the free list.
*/

OSErr DoCloseSession(SessionPtr session) {
	auto	OSErr				osErr;
	
	if (nil != (*session).alias)
		DisposeHandle((Handle) (*session).alias);
	osErr = SessionClose(session);
	FreeSession(session);
	return osErr;
}

OSErr MenuAdjust(WindowPtr window) {
	auto	OSErr				osErr		= noErr;
	auto	SessionPtr			session		= nil;
	auto	MenuHandle			menu;
	auto	CMBufferSizes		size;
	auto	CMStatFlags			status;
			
	if (IsSessionWindow(window))
		session = (SessionPtr) window;
		
	menu = GetMHandle(mFile);
	DisableItem(menu, iNew);
	DisableItem(menu, iOpen);
	DisableItem(menu, iClose);
	DisableItem(menu, iSave);
	DisableItem(menu, iSaveAs);
	DisableItem(menu, iPreferences);
	if (gAvailSessionList) {
		EnableItem(menu, iNew);
		EnableItem(menu, iOpen);
	}
	if (nil != session) {
		EnableItem(menu, iClose);
		if ((*session).alias)
			EnableItem(menu, iSave);
		EnableItem(menu, iSaveAs);
		EnableItem(menu, iPreferences);
	} else if (IsDescAccWindow(window))
		EnableItem(menu, iClose);
	EnableItem(menu, iQuit);
	
	menu = GetMHandle(mEdit);
	DisableItem(menu, iUndo);
	DisableItem(menu, iCut);
	DisableItem(menu, iCopy);
	DisableItem(menu, iClear);
	DisableItem(menu, iPaste);
	if ( IsDescAccWindow(window) ) {
		EnableItem(menu, iUndo);
		EnableItem(menu, iCut);
		EnableItem(menu, iCopy);
		EnableItem(menu, iClear);
		EnableItem(menu, iPaste);
	}
	
	menu = GetMHandle(mSession);
	DisableItem(menu, iConnect);
	DisableItem(menu, iWaitConnect);
	DisableItem(menu, iDisconnect);
	DisableItem(menu, iSendFile);
	DisableItem(menu, iReceiveFile);
	DisableItem(menu,  iResetConnection);
	DisableItem(menu,  iResetTerminal);
	DisableItem(menu,  iSendBreak);
	if (nil != session) {
		if (noErr == (osErr = CMStatus((*session).connection, size, &status))) {
			if ((status & cmStatusOpening) || (status & cmStatusListenPend)) {
				EnableItem(menu,  iDisconnect);
			} else if ((status & cmStatusOpen)) {
				EnableItem(menu,  iDisconnect);
				EnableItem(menu,  iResetConnection);
				EnableItem(menu,  iResetTerminal);
				EnableItem(menu,  iSendBreak);
				if (0 == (ftSendDisable & (**(*session).fileTransfer).attributes))
					EnableItem(menu, iSendFile);
				if (0 == (ftReceiveDisable & (**(*session).fileTransfer).attributes))
					EnableItem(menu,  iReceiveFile);
			} else {
				EnableItem(menu,  iConnect);
				EnableItem(menu,  iWaitConnect);
			}
		}
	}
	
	menu = GetMHandle(mSettings);
	DisableItem(menu, iSetConnection);
	DisableItem(menu, iSetTerminal);
	DisableItem(menu, iSetFileTransfer);
	if (nil != session) {
		EnableItem(menu, iSetConnection);
		EnableItem(menu, iSetTerminal);
		EnableItem(menu, iSetFileTransfer);
	}
	
	return osErr;
}

/*
**	MenuCommand processes the menu commands when selected by the user through mouse
**	or keyboard input. It does make some assumptions about the validity of being called
**	to perform desired actions. This is held in check by the MenuAdjust function.
*/
OSErr MenuCommand(short menuID, short itemID, WindowPtr window) {
	auto	OSErr				osErr		= noErr;
	auto	Str255				daName;
	auto	short				daRefNum;
	auto	StandardFileReply	reply;
	auto	SFTypeList			types		= { kDocFileType };
	auto	short				numTypes	= 1;
	auto	Boolean				changed;
	auto	SessionPtr			session		= nil;
	auto	FSSpec				targFSSpec;
	auto	MenuHandle			menu;
	
	switch (menuID) {
		case mApple :
			switch (itemID) {
				case iAbout :
					if (nil == gSplash) {
						gSplash = GetNewDialog(rAboutDLOG, 0, (WindowPtr) -1);
						DrawDialog(gSplash);
						HMSetMenuResID(mApple, 256);
						if (nil != (menu = GetMHandle(mApple)))
							DisableItem(menu, 0);
						HMSetMenuResID(mFile, 256);
						if (nil != (menu = GetMHandle(mFile)))
							DisableItem(menu, 0);
						HMSetMenuResID(mEdit, 256);
						if (nil != (menu = GetMHandle(mEdit)))
							DisableItem(menu, 0);
						HMSetMenuResID(mSession, 256);
						if (nil != (menu = GetMHandle(mSession)))
							DisableItem(menu, 0);
						HMSetMenuResID(mSettings, 256);
						if (nil != (menu = GetMHandle(mSettings)))
							DisableItem(menu, 0);
					//	if (noErr == HMGetHelpMenuHandle(&menu))
					//		DisableItem(menu, 1);
						DrawMenuBar();
					}
					break;
				default :
					GetItem(GetMHandle(mApple), itemID, daName);
					daRefNum = OpenDeskAcc(daName); 
					break;
			}
			break;
		case mFile:
			if (nil != window)
				if (IsSessionWindow(window))
					session = (SessionPtr) window;
			switch (itemID) {
				case iNew :
					if (noErr == (osErr = DoPreferedSession(&session)))
						InsertSession(session);
					else {
						ErrorAlert(osErr, "\pA default session is being created");
						(void) DoCloseSession(session);
						if (noErr == (osErr = DoNewSession(&session)))
							InsertSession(session);
						else {
							ErrorAlert(osErr, "\pNo session could be created");
							(void) DoCloseSession(session);
						}
					}
					break;
				case iOpen :
					StandardGetFile(nil, numTypes, types, &reply);
					if (reply.sfGood)
						if (noErr == (osErr = DoOpenSession(&session, &reply.sfFile, 0 != (isStationary & reply.sfFlags))))
							InsertSession(session);
						else
							(void) DoCloseSession(session);
					break;
				case iSave:
					if (noErr == (osErr = ResolveAlias(nil, (*session).alias, &targFSSpec, &changed)))
						osErr = DoSafeSaveSession(session, &targFSSpec);
					break;
				case iSaveAs:
					StandardPutFile("\pSave as what name?", "\puntitled", &reply);
					if (reply.sfGood) {
						if (reply.sfReplacing)
							osErr = DoSafeSaveSession(session, &reply.sfFile);
						else
							osErr = DoSaveSession(session, &reply.sfFile);
						if (noErr == osErr) {
							SetWTitle((WindowPtr) &(*session).window, reply.sfFile.name);
							if ((*session).alias)
								DisposeHandle((Handle) (*session).alias);
							osErr = NewAlias(nil, &reply.sfFile, &(*session).alias);
						}
					}
					break;
				case iClose :
					if (session) {
						DeleteSession(session);
						osErr = DoCloseSession(session);
					} else if (IsDescAccWindow(window))
						CloseDeskAcc(((WindowPeek) window)->windowKind);
					break;
				case iPreferences :
					if (nil != session)
						if (noErr == (osErr = FindPreferenceFile(&targFSSpec)))
							osErr = DoSafeSaveSession(session, &targFSSpec);
					break;
				case iQuit :
					while (nil != gWorkSessionList) {
						session = gWorkSessionList;
						DeleteSession(session);
						osErr = DoCloseSession(session);
					}
					ExitToShell();
					break;
			}
			break;
		case mEdit:
			switch (itemID) {
				case iUndo :
				case iCut :
				case iCopy :
				case iPaste :
				case iClear :
					SystemEdit(itemID-1);
					break;
			}
			break;
		case mSession:
			if (nil != window) {
				if (IsSessionWindow(window)) {
					session = (SessionPtr) window;
					switch (itemID) {
						case iConnect :
							osErr = ConnectionConnect((*session).connection);
							break;
						case iWaitConnect :
							osErr = ConnectionListen((*session).connection);
							break;
						case iDisconnect :
							osErr = ConnectionDisconnect((*session).connection);
							break;
						case iSendFile :
							types[0] = 'TEXT';
							numTypes = ((**(*session).fileTransfer).attributes & ftTextOnly) ? 1 : -1;
							StandardGetFile(nil, numTypes, types, &reply);
							if (reply.sfGood)
								osErr = FileTransferSend((*session).fileTransfer, &reply.sfFile);
							break;
						case iReceiveFile :
							FileTransferReceive((*session).fileTransfer);
							break;
						case iResetConnection :
							CMReset((*session).connection);
							break;
						case iResetTerminal :
							TMReset((*session).terminal);
							break;
						case iSendBreak :
							CMBreak((*session).connection, 60, false, nil);
							break;
					}
				}
			}
			break;
		case mSettings :
			if (IsSessionWindow(window)) {
				session = (SessionPtr) window;
				FileTransferAutoOff((*session).fileTransfer);
				switch (itemID) {
					case iSetConnection :
						osErr = CMChoose(&(*session).connection, gChooseWhere, nil);
						break;
					case iSetTerminal :
						osErr = TMChoose(&(*session).terminal, gChooseWhere, nil);
						break;
					case iSetFileTransfer :
						osErr = FTChoose(&(*session).fileTransfer, gChooseWhere, nil);
						break;
				}
				osErr = noErr;
				FileTransferAutoOn((*session).fileTransfer);
			}
			break;
		default :
			if (IsSessionWindow(window)) {
				session = (SessionPtr) window;
				if (! CMMenu((*session).connection, menuID, itemID)) 
					if (! TMMenu((*session).terminal, menuID, itemID)) 
						FTMenu((*session).fileTransfer, menuID, itemID);
			}
			break;

	}
	HiliteMenu(0);
	
	return osErr;
}
